/*
  Liquid War 6 is a unique multiplayer wargame.
  Copyright (C)  2005, 2006, 2007, 2008  Christian Mauduit <ufoot@ufoot.org>

  This program is free software; you can redistribute it and/or modify
  it under the terms of the GNU General Public License as published by
  the Free Software Foundation, either version 3 of the License, or
  (at your option) any later version.

  This program is distributed in the hope that it will be useful,
  but WITHOUT ANY WARRANTY; without even the implied warranty of
  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
  GNU General Public License for more details.

  You should have received a copy of the GNU General Public License
  along with this program.  If not, see <http://www.gnu.org/licenses/>.


  Liquid War 6 homepage : http://www.gnu.org/software/liquidwar6/
  Contact author        : ufoot@ufoot.org
*/

#ifndef LIQUIDWAR6GFX_MOD_GL_BACKGROUND_BUBBLES_INTERNAL_H
#define LIQUIDWAR6GFX_MOD_GL_BACKGROUND_BUBBLES_INTERNAL_H

#include <GL/gl.h>
#include <SDL/SDL.h>
#include <SDL/SDL_ttf.h>

#include "gl-bubbles.h"

#define _MOD_GL_BACKGROUND_BUBBLES_MAX_BUBBLES 64

typedef struct _mod_gl_background_bubbles_bubble_s
{
  int active;
  int t0;
  float size;
  float x;
}
_mod_gl_background_bubbles_bubble_t;

typedef struct _mod_gl_background_bubbles_state_s
{
  _mod_gl_background_bubbles_bubble_t
    bubbles[_MOD_GL_BACKGROUND_BUBBLES_MAX_BUBBLES];
}
_mod_gl_background_bubbles_state_t;

typedef struct _mod_gl_background_bubbles_image_data_s
{
  SDL_Surface *background;
  SDL_Surface *bubble;
}
_mod_gl_background_bubbles_image_data_t;

typedef struct _mod_gl_background_bubbles_texture_data_s
{
  GLuint background;
  GLuint bubble;
}
_mod_gl_background_bubbles_texture_data_t;

typedef struct _mod_gl_background_bubbles_const_data_s
{
  int nb_bubbles;
  float yspeed;
  float bubble_yspeed;
  float bubble_size_min;
  float bubble_size_max;
  lw6sys_color_f_t menu_fg;
  lw6sys_color_f_t menu_bg;
  lw6sys_color_f_t menu_fg_disabled;
  lw6sys_color_f_t menu_bg_disabled;
  lw6sys_color_f_t menu_fg_selected;
  lw6sys_color_f_t menu_bg_selected;
}
_mod_gl_background_bubbles_const_data_t;

typedef struct _mod_gl_background_bubbles_context_s
{
  _mod_gl_background_bubbles_state_t state;
  _mod_gl_background_bubbles_image_data_t image_data;
  _mod_gl_background_bubbles_texture_data_t texture_data;
  _mod_gl_background_bubbles_const_data_t const_data;
}
_mod_gl_background_bubbles_context_t;

/*
 * In display.c
 */
extern void _mod_gl_background_bubbles_display (mod_gl_utils_context_t *
						utils_context,
						_mod_gl_background_bubbles_context_t
						* bubbles_context,
						lw6gui_look_t * look);

/*
 * In data.c
 */
extern int _mod_gl_background_bubbles_load_data (mod_gl_utils_context_t *
						 utils_context,
						 _mod_gl_background_bubbles_context_t
						 * bubbles_context);
extern void _mod_gl_background_bubbles_unload_data (mod_gl_utils_context_t *
						    utils_context,
						    _mod_gl_background_bubbles_context_t
						    * bubbles_context);

/*
 * In setup.c
 */
extern _mod_gl_background_bubbles_context_t
  * _mod_gl_background_bubbles_init (mod_gl_utils_context_t * utils_context);
extern void _mod_gl_background_bubbles_quit (mod_gl_utils_context_t *
					     utils_context,
					     _mod_gl_background_bubbles_context_t
					     * bubbles_context);

#endif
