
/*
  Liquid War 6 is a unique multiplayer wargame.
  Copyright (C)  2005, 2006, 2007, 2008  Christian Mauduit <ufoot@ufoot.org>

  This program is free software; you can redistribute it and/or modify
  it under the terms of the GNU General Public License as published by
  the Free Software Foundation, either version 3 of the License, or
  (at your option) any later version.

  This program is distributed in the hope that it will be useful,
  but WITHOUT ANY WARRANTY; without even the implied warranty of
  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
  GNU General Public License for more details.

  You should have received a copy of the GNU General Public License
  along with this program.  If not, see <http://www.gnu.org/licenses/>.
  

  Liquid War 6 homepage : http://www.gnu.org/software/liquidwar6/
  Contact author        : ufoot@ufoot.org
*/

#include "config.h"
#include "gl-menu.h"

static void
warning (char *func_name)
{
  lw6sys_log (LW6SYS_LOG_WARNING,
	      _("menu backend function \"%s\" is not defined"));
}

int
mod_gl_menu_init (mod_gl_utils_context_t * utils_context,
		  mod_gl_menu_backend_t * backend)
{
  if (backend->init)
    {
      backend->menu_context = backend->init (utils_context);
    }
  else
    {
      warning ("menu_init");
    }

  return backend->menu_context ? 1 : 0;
}

void
mod_gl_menu_quit (mod_gl_utils_context_t * utils_context,
		  mod_gl_menu_backend_t * backend)
{
  if (backend->quit)
    {
      backend->quit (utils_context, backend->menu_context);
    }
  else
    {
      warning ("menu_quit");
    }
}

void
mod_gl_menu_display (mod_gl_utils_context_t * utils_context,
		     mod_gl_menu_backend_t * backend, lw6gui_look_t * look,
		     lw6gui_menu_t * menu)
{
  if (backend->display)
    {
      backend->display (utils_context, backend->menu_context, look, menu);
    }
  else
    {
      warning ("menu_display");
    }
}

void
mod_gl_menu_pick_item (mod_gl_utils_context_t * utils_context,
		       mod_gl_menu_backend_t * backend, lw6gui_look_t * look,
		       int *position,
		       int *scroll, lw6gui_menu_t * menu, int screen_x,
		       int screen_y)
{
  if (backend->pick_item)
    {
      backend->pick_item (utils_context, backend->menu_context, look,
			  position, scroll, menu, screen_x, screen_y);
    }
  else
    {
      warning ("menu_pick_item");
    }
}

int
mod_gl_menu_warp_mouse_on_menuitem (mod_gl_utils_context_t * utils_context,
				    mod_gl_menu_backend_t * backend,
				    lw6gui_look_t * look,
				    lw6gui_menu_t * menu, int position)
{
  int ret = 0;

  if (backend->warp_mouse_on_menuitem)
    {
      ret =
	backend->warp_mouse_on_menuitem (utils_context, backend->menu_context,
					 look, menu, position);
    }
  else
    {
      warning ("menu_warp_mouse_on_menuitem");
    }

  return ret;
}
