
/*
  Liquid War 6 is a unique multiplayer wargame.
  Copyright (C)  2005, 2006, 2007, 2008  Christian Mauduit <ufoot@ufoot.org>

  This program is free software; you can redistribute it and/or modify
  it under the terms of the GNU General Public License as published by
  the Free Software Foundation, either version 3 of the License, or
  (at your option) any later version.

  This program is distributed in the hope that it will be useful,
  but WITHOUT ANY WARRANTY; without even the implied warranty of
  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
  GNU General Public License for more details.

  You should have received a copy of the GNU General Public License
  along with this program.  If not, see <http://www.gnu.org/licenses/>.
  

  Liquid War 6 homepage : http://www.gnu.org/software/liquidwar6/
  Contact author        : ufoot@ufoot.org
*/

#include <stdio.h>
#include <string.h>
#include <time.h>
#include <string.h>

#include "config.h"
#include "../gfx.h"
#include "mod-gl-internal.h"

int
_mod_gl_set_backends (_mod_gl_context_t * gfx_context, lw6gui_look_t * look)
{
  int ret = 1;

  ret =
    _mod_gl_set_background_backend (gfx_context, look->style.background_style)
    && ret;
  ret = _mod_gl_set_hud_backend (gfx_context, look->style.hud_style) && ret;
  ret = _mod_gl_set_menu_backend (gfx_context, look->style.menu_style) && ret;
  ret = _mod_gl_set_view_backend (gfx_context, look->style.view_style) && ret;

  return ret;
}

static void
clear_background_backend (_mod_gl_context_t * gfx_context)
{
  if (gfx_context->background_backend)
    {
      mod_gl_background_quit (&(gfx_context->utils_context),
			      gfx_context->background_backend);
      mod_gl_background_destroy_backend (&(gfx_context->utils_context),
					 gfx_context->background_backend);
      gfx_context->background_backend = NULL;
    }

  mod_gl_utils_clear_texturecache (&(gfx_context->utils_context));
  mod_gl_utils_clear_menucache (&(gfx_context->utils_context));
}

static void
clear_hud_backend (_mod_gl_context_t * gfx_context)
{
  if (gfx_context->hud_backend)
    {
      mod_gl_hud_quit (&(gfx_context->utils_context),
		       gfx_context->hud_backend);
      mod_gl_hud_destroy_backend (&(gfx_context->utils_context),
				  gfx_context->hud_backend);
      gfx_context->hud_backend = NULL;
    }

  mod_gl_utils_clear_texturecache (&(gfx_context->utils_context));
  mod_gl_utils_clear_menucache (&(gfx_context->utils_context));
}

static void
clear_menu_backend (_mod_gl_context_t * gfx_context)
{
  if (gfx_context->menu_backend)
    {
      mod_gl_menu_quit (&(gfx_context->utils_context),
			gfx_context->menu_backend);
      mod_gl_menu_destroy_backend (&(gfx_context->utils_context),
				   gfx_context->menu_backend);
      gfx_context->menu_backend = NULL;
    }

  mod_gl_utils_clear_texturecache (&(gfx_context->utils_context));
  mod_gl_utils_clear_menucache (&(gfx_context->utils_context));
}

static void
clear_view_backend (_mod_gl_context_t * gfx_context)
{
  if (gfx_context->view_backend)
    {
      mod_gl_view_quit (&(gfx_context->utils_context),
			gfx_context->view_backend);
      mod_gl_view_destroy_backend (&(gfx_context->utils_context),
				   gfx_context->view_backend);
      gfx_context->view_backend = NULL;
    }

  mod_gl_utils_clear_texturecache (&(gfx_context->utils_context));
  mod_gl_utils_clear_menucache (&(gfx_context->utils_context));
}

int
_mod_gl_set_background_backend (_mod_gl_context_t * gfx_context,
				char *background_style)
{
  int ret = 0;

  if ((!background_style) || strlen (background_style) == 0)
    {
      background_style = LW6MAP_STYLE_DEFAULT_BACKGROUND_STYLE;
    }

  if ((!gfx_context->background_backend)
      || strcmp (gfx_context->background_backend->backend_name,
		 background_style))
    {
      clear_background_backend (gfx_context);

      gfx_context->background_backend =
	mod_gl_background_create_backend (&(gfx_context->utils_context),
					  background_style);
      if (gfx_context->background_backend)
	{
	  ret =
	    mod_gl_background_init (&(gfx_context->utils_context),
				    gfx_context->background_backend);
	}
    }

  return ret;
}

int
_mod_gl_set_hud_backend (_mod_gl_context_t * gfx_context, char *hud_style)
{
  int ret = 0;

  if ((!hud_style) || strlen (hud_style) == 0)
    {
      hud_style = LW6MAP_STYLE_DEFAULT_HUD_STYLE;
    }

  if ((!gfx_context->hud_backend)
      || strcmp (gfx_context->hud_backend->backend_name, hud_style))
    {
      clear_hud_backend (gfx_context);

      gfx_context->hud_backend =
	mod_gl_hud_create_backend (&(gfx_context->utils_context), hud_style);
      if (gfx_context->hud_backend)
	{
	  ret =
	    mod_gl_hud_init (&(gfx_context->utils_context),
			     gfx_context->hud_backend);
	}
    }

  return ret;
}

int
_mod_gl_set_menu_backend (_mod_gl_context_t * gfx_context, char *menu_style)
{
  int ret = 0;

  if ((!menu_style) || strlen (menu_style) == 0)
    {
      menu_style = LW6MAP_STYLE_DEFAULT_MENU_STYLE;
    }

  if ((!gfx_context->menu_backend)
      || strcmp (gfx_context->menu_backend->backend_name, menu_style))
    {
      clear_menu_backend (gfx_context);

      gfx_context->menu_backend =
	mod_gl_menu_create_backend (&(gfx_context->utils_context),
				    menu_style);
      if (gfx_context->menu_backend)
	{
	  ret =
	    mod_gl_menu_init (&(gfx_context->utils_context),
			      gfx_context->menu_backend);
	}
    }

  return ret;
}

int
_mod_gl_set_view_backend (_mod_gl_context_t * gfx_context, char *view_style)
{
  int ret = 0;

  if ((!view_style) || strlen (view_style) == 0)
    {
      view_style = LW6MAP_STYLE_DEFAULT_VIEW_STYLE;
    }

  if ((!gfx_context->view_backend)
      || strcmp (gfx_context->view_backend->backend_name, view_style))
    {
      clear_view_backend (gfx_context);

      gfx_context->view_backend =
	mod_gl_view_create_backend (&(gfx_context->utils_context),
				    view_style);
      if (gfx_context->view_backend)
	{
	  ret =
	    mod_gl_view_init (&(gfx_context->utils_context),
			      gfx_context->view_backend);
	}
    }

  return ret;
}

/*
 * Low-level SDL initialisation.
 */
_mod_gl_context_t *
_mod_gl_init (int argc, char *argv[], int width, int height, int fullscreen,
	      void (*resize_callback) (int width, int height, int fullscreen),
	      int ticks)
{
  _mod_gl_context_t *gfx_context = NULL;
  int background_ok = 0;
  int hud_ok = 0;
  int menu_ok = 0;
  int view_ok = 0;
  int sdl_ok = 1;
  int ttf_ok = 1;
  lw6map_style_t map_style;

  gfx_context =
    (_mod_gl_context_t *) LW6SYS_CALLOC (sizeof (_mod_gl_context_t));
  if (gfx_context)
    {
      if (mod_gl_utils_path_init (&(gfx_context->utils_context), argc, argv))
	{

	  if (lw6sys_sdl_register ())
	    {
	      sdl_ok = !SDL_Init (0);
	    }

	  sdl_ok = sdl_ok && (SDL_WasInit (SDL_INIT_TIMER)
			      || !SDL_InitSubSystem (SDL_INIT_TIMER));
	  sdl_ok = sdl_ok && (SDL_WasInit (SDL_INIT_VIDEO)
			      || !SDL_InitSubSystem (SDL_INIT_VIDEO));

	  if (sdl_ok)
	    {
	      lw6sys_log (LW6SYS_LOG_INFO, _("SDL Init"));
	    }
	  else
	    {
	      lw6sys_log (LW6SYS_LOG_ERROR, _("SDL init error: \"%s\""),
			  SDL_GetError ());
	    }

	  SDL_EnableUNICODE (1);

	  ttf_ok = (TTF_Init () != -1);
	  if (ttf_ok)
	    {
	      lw6sys_log (LW6SYS_LOG_INFO, _("SDL_ttf Init"));
	    }
	  else
	    {
	      lw6sys_log (LW6SYS_LOG_ERROR,
			  _("SDL_ttf init error: \"%s\""), TTF_GetError ());
	    }

	  srandom (time (NULL));

	  if (sdl_ok && ttf_ok)
	    {
	      if (mod_gl_utils_set_video_mode
		  (&(gfx_context->utils_context), width, height, fullscreen,
		   resize_callback))
		{
		  if (mod_gl_utils_load_data (&(gfx_context->utils_context)))
		    {
		      memset (&map_style, 0, sizeof (lw6map_style_t));
		      lw6map_style_defaults (&map_style);
		      background_ok =
			_mod_gl_set_background_backend (gfx_context,
							map_style.
							background_style);
		      hud_ok =
			_mod_gl_set_hud_backend (gfx_context,
						 map_style.hud_style);
		      menu_ok =
			_mod_gl_set_menu_backend (gfx_context,
						  map_style.menu_style);
		      view_ok =
			_mod_gl_set_view_backend (gfx_context,
						  map_style.view_style);
		      mod_gl_utils_update_team_color_map (&
							  (gfx_context->
							   utils_context.
							   team_color_map),
							  &map_style);
		      lw6map_style_clear (&map_style);
		    }
		  else
		    {
		      lw6sys_log (LW6SYS_LOG_ERROR, _("unable to load data"));
		    }
		}
	      else
		{
		  lw6sys_log (LW6SYS_LOG_ERROR,
			      _("unable to set video mode"));
		}
	    }

	  if (gfx_context)
	    {
	      gfx_context->utils_context.joysticks_state.supported =
		!SDL_InitSubSystem (SDL_INIT_JOYSTICK);
	      if (gfx_context->utils_context.joysticks_state.supported)
		{
		  mod_gl_utils_joystick_init (&
					      (gfx_context->utils_context.
					       joysticks_state));
		}
	      else
		{
		  lw6sys_log (LW6SYS_LOG_NOTICE, _("no joystick support"));
		}
	    }

	  if (gfx_context && !(background_ok && hud_ok && menu_ok && view_ok))
	    {
	      _mod_gl_quit (gfx_context);
	      gfx_context = NULL;
	    }

	  mod_gl_utils_timer_init (&(gfx_context->utils_context), ticks);
	}
      else
	{
	  LW6SYS_FREE (gfx_context);
	  gfx_context = NULL;
	}
    }

  return gfx_context;
}

/*
 * Ends-up all SDL stuff.
 */
void
_mod_gl_quit (_mod_gl_context_t * gfx_context)
{
  float quit_sleep;

  /*
   * Keep this value locally since it can disappear
   * when freeing stuff.
   */
  quit_sleep = gfx_context->utils_context.const_data.quit_sleep;

  if (gfx_context->utils_context.joysticks_state.supported)
    {
      mod_gl_utils_joystick_quit (&
				  (gfx_context->utils_context.
				   joysticks_state));
    }

  clear_background_backend (gfx_context);
  clear_hud_backend (gfx_context);
  clear_menu_backend (gfx_context);
  clear_view_backend (gfx_context);

  mod_gl_utils_unload_data (&(gfx_context->utils_context));

  mod_gl_utils_path_quit (&(gfx_context->utils_context));

  glFinish ();
  lw6sys_log (LW6SYS_LOG_INFO, _("SDL_ttf Quit"));
  TTF_Quit ();

  SDL_QuitSubSystem (SDL_INIT_VIDEO);
  SDL_QuitSubSystem (SDL_INIT_TIMER);

  if (lw6sys_sdl_unregister ())
    {
      lw6sys_log (LW6SYS_LOG_INFO, _("SDL Quit"));
      SDL_Quit ();
    }

  /*
   * For some reason, I suspect some segfaults occur when
   * "dlclosing" mod-gl just after SDL_Quit. Might be a handler
   * or callback called afterwards, whatever. So I prefer
   * "wasting" a little time when closing, one never knows,
   * it might better things.
   */
  lw6sys_sleep (quit_sleep);

  if (gfx_context->utils_context.surface_counter.new_counter !=
      gfx_context->utils_context.surface_counter.delete_counter)
    {
      lw6sys_log (LW6SYS_LOG_WARNING,
		  _
		  ("possible memory leak, %d calls to SDL_CreateSurface, IMG_Load or TTF_RenderUTF8, and %d calls to SDL_FreeSurface"),
		  gfx_context->utils_context.surface_counter.
		  new_counter,
		  gfx_context->utils_context.surface_counter.delete_counter);
    }

  if (gfx_context->utils_context.texture_counter.new_counter !=
      gfx_context->utils_context.texture_counter.delete_counter)
    {
      lw6sys_log (LW6SYS_LOG_WARNING,
		  _
		  ("possible memory leak, %d calls to glGenTexture and %d calls to glDeleteTextures"),
		  gfx_context->utils_context.texture_counter.
		  new_counter,
		  gfx_context->utils_context.texture_counter.delete_counter);
    }

  LW6SYS_FREE (gfx_context);
}
