
/*
  Liquid War 6 is a unique multiplayer wargame.
  Copyright (C)  2005, 2006, 2007, 2008  Christian Mauduit <ufoot@ufoot.org>

  This program is free software; you can redistribute it and/or modify
  it under the terms of the GNU General Public License as published by
  the Free Software Foundation, either version 3 of the License, or
  (at your option) any later version.

  This program is distributed in the hope that it will be useful,
  but WITHOUT ANY WARRANTY; without even the implied warranty of
  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
  GNU General Public License for more details.

  You should have received a copy of the GNU General Public License
  along with this program.  If not, see <http://www.gnu.org/licenses/>.
  

  Liquid War 6 homepage : http://www.gnu.org/software/liquidwar6/
  Contact author        : ufoot@ufoot.org
*/

#include <string.h>

#include "config.h"
#include "srv.h"
#include "mod-tcpd/mod-tcpd.h"
#include "mod-udpd/mod-udpd.h"
#include "mod-httpd/mod-httpd.h"

lw6sys_assoc_t *
lw6srv_get_backends (int argc, char *argv[])
{
  lw6sys_assoc_t *ret = NULL;

  ret = lw6sys_assoc_new (NULL);
  if (ret)
    {
      lw6sys_assoc_set (&ret, "tcpd", (void *) _("TCP"));
      lw6sys_assoc_set (&ret, "udpd", (void *) _("UDP"));
      lw6sys_assoc_set (&ret, "httpd", (void *) _("HTTP"));
    }

  return ret;
}

lw6srv_backend_t *
lw6srv_create_backend (int argc, char *argv[], char *name)
{
  lw6srv_backend_t *backend = NULL;

  if (name && !strcmp (name, "tcpd"))
    {
      backend = mod_tcpd_create_backend ();
    }
  if (name && !strcmp (name, "udpd"))
    {
      backend = mod_udpd_create_backend ();
    }
  if (name && !strcmp (name, "httpd"))
    {
      backend = mod_httpd_create_backend ();
    }

  if (backend)
    {
      backend->argc = argc;
      backend->argv = argv;
    }
  else
    {
      lw6sys_log (LW6SYS_LOG_WARNING,
		  _("server backend \"%s\" does not exist"), name);
    }

  return backend;
}

void
lw6srv_destroy_backend (lw6srv_backend_t * backend)
{
  LW6SYS_FREE (backend);
}
