/*
  Liquid War 6 is a unique multiplayer wargame.
  Copyright (C)  2005, 2006, 2007, 2008  Christian Mauduit <ufoot@ufoot.org>

  This program is free software; you can redistribute it and/or modify
  it under the terms of the GNU General Public License as published by
  the Free Software Foundation, either version 3 of the License, or
  (at your option) any later version.

  This program is distributed in the hope that it will be useful,
  but WITHOUT ANY WARRANTY; without even the implied warranty of
  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
  GNU General Public License for more details.

  You should have received a copy of the GNU General Public License
  along with this program.  If not, see <http://www.gnu.org/licenses/>.


  Liquid War 6 homepage : http://www.gnu.org/software/liquidwar6/
  Contact author        : ufoot@ufoot.org
*/

#include <stdio.h>
#include <string.h>

#include "config.h"
#include "sys.h"

#define TEST_KEYWORD_AS "---this-is_a_TEST-KeYWord"
#define TEST_ARG_KEYWORD "my-option"
#define TEST_ARG_ARGV_STRING "--my-option=my-value"
#define CHECKSUM_INT32_DATA 1234
#define CHECKSUM_INT32_RESULT 0x5d3d512e
#define COLOR_ASCII "#12345678"
#define COLOR_8_R 0x12
#define COLOR_8_G 0x34
#define COLOR_8_B 0x56
#define COLOR_8_A 0x78
#define PATH_WITH_SLASH "/foo/bar/"
#define PATH_WITHOUT_SLASH "/foo/bar"
#define PATH_CONCAT "/foo/bar/foo/bar"
#define PATH_CONCAT_PARENT "/foo/bar/foo"
#define PATH_RELATIVE "../../foo/bar"
#define PATH_RELATIVE_UNPARENT "../foo/bar"
#define MALLOC_SIZE 1048576
#define THREAD_N 5
#define THREAD_SLEEP_MAIN 0.66f
#define THREAD_SLEEP_CALLBACK 0.2f
#define THREAD_TEXT "foo bar"
#define SLEEP_TIME 3
#define BLANK_STR "\t "
#define ASCII_STR "foo,bar"
#define TEST_ARGC 4
#define TEST_ARGV0 "test.out"
#define TEST_ARGV1 "--prefix=/toto"
#define TEST_ARGV2 "-map-dir=/home/user/my-maps/"
#define TEST_ARGV3 "THISisAbug!==--foobar=123---?\\"
#define TEST_FILENAME "/tmp/this-is-a-test.txt"
#define TEST_CONTENT "This is a test"
#define TEST_HEXA_INT64 1
#define TEST_HEXA_INT32 123456789
#define TEST_HEXA_INT16 12345
#define TEST_HEXA_INT8 123
#define TEST_HEXA_FLOAT 2.0f
#define TEST_HEXA_STR1 "abc"
#define TEST_HEXA_STR2 "defghi"
#define TEST_HEXA_WH {1,2}
#define TEST_HEXA_XY {3,4}
#define TEST_HEXA_COLOR {25,50,75,100}
#define TEST_KEYWORD "prefix"
#define TEST_ENV_CONCAT1 "/my/path/1"
#define TEST_ENV_CONCAT2 "/my/path/2"
#define TEST_ENV_PUTENV "LW6_MY_KEY=my-value"
#define TEST_ENV_KEYWORD "my-key"
#define TEST_ENV_SPLIT "foo1/foo2;foo3:bar1;bar2/bar3"
#define TEST_REFORMAT_STR1 "this is a short string"
#define TEST_REFORMAT_STR2 "this is a very long string it should be cut in some places but wherever there is a very-very-very-very-very-very-very-very-very-very-very-very-very-very-very-very-very-very long word it won't be cut.\n\nIt should even handle newlines in some places but well, this is not absolutely necessary"
#define TEST_REFORMAT_PREFIX "# "
#define TEST_REFORMAT_COLUMNS 40
#define TEST_SERIALIZE 12345
#define TEST_SORT_LENGTH 10
#define TEST_SORT_INT {9,3,4,5,6,8,7,2,0,1};
#define TEST_SORT_INT_MIN 0
#define TEST_SORT_INT_MAX 9
#define TEST_SORT_FLOAT {9.0,3.0,4.0,5.0,6.0,8.0,7.0,2.0,0.0,1.0};
#define TEST_SORT_FLOAT_MIN 0.0
#define TEST_SORT_FLOAT_MAX 9.0
#define TEST_SORT_STR {"9","3","4","5","6","8","7","2","0","1"};
#define TEST_SORT_STR_MIN "0"
#define TEST_SORT_STR_MAX "9"
#define TEST_CONVERT_INT 421
#define TEST_CONVERT_BOOL 1
#define TEST_CONVERT_FLOAT 3.14f

/*
 * Testing functions in arg.c
 */
static int
test_arg ()
{
  int ret = 1;
  LW6SYS_TEST_FUNCTION_BEGIN;

  {
    char *value = NULL;
    int argc = TEST_ARGC;
    char *argv[TEST_ARGC] =
      { TEST_ARGV0, TEST_ARGV1, TEST_ARGV2, TEST_ARGV3 };

    lw6sys_log (LW6SYS_LOG_DEBUG, _("trying to match \"%s\""),
		TEST_ARG_ARGV_STRING);
    if (lw6sys_arg_match (TEST_ARG_KEYWORD, TEST_ARG_ARGV_STRING))
      {
	// does match!
      }
    else
      {
	lw6sys_log (LW6SYS_LOG_WARNING,
		    _
		    ("command line arg \"%s\" does not match keyword \"%s\""),
		    TEST_ARG_ARGV_STRING, TEST_ARG_KEYWORD);
	ret = 0;
      }

    if (lw6sys_arg_exists (argc, argv, TEST_KEYWORD))
      {
	lw6sys_log (LW6SYS_LOG_DEBUG, _("keyword \"%s\" is present"),
		    TEST_KEYWORD);
      }
    else
      {
	lw6sys_log (LW6SYS_LOG_WARNING, _("keyword \"%s\" not present"),
		    TEST_KEYWORD);
	ret = 0;
      }

    value = lw6sys_arg_get_value (argc, argv, TEST_KEYWORD);
    if (value)
      {
	lw6sys_log (LW6SYS_LOG_DEBUG, _("value for \"%s\" is \"%s\""),
		    TEST_KEYWORD, value);
	LW6SYS_FREE (value);
      }
    else
      {
	lw6sys_log (LW6SYS_LOG_WARNING, _("no value for keyword \"%s\""),
		    TEST_KEYWORD);
	ret = 0;
      }
  }

  LW6SYS_TEST_FUNCTION_END;
  return ret;
}

/*
 * Used to test lw6sys_assoc_map.
 */
static void
assoc_map_func (void *func_data, char *key, void *value)
{
  int *ret = (int *) func_data;
  int *i = (int *) value;

  lw6sys_log (LW6SYS_LOG_DEBUG,
	      _("assoc item ret=%d, key=\"%s\", value=%d"), *ret, key, *i);
}

/*
 * Testing functions in assoc.c
 */
static int
test_assoc ()
{
  int ret = 1;
  lw6sys_assoc_t *assoc;
  lw6sys_assoc_t *assoc_copy;
  LW6SYS_TEST_FUNCTION_BEGIN;

  {
    lw6sys_log (LW6SYS_LOG_DEBUG, _("new/delete on assoc"));
    assoc = lw6sys_assoc_new (NULL);
    lw6sys_assoc_free (assoc);
  }

  {
    int a = 3, b = 5, c = 7;

    lw6sys_log (LW6SYS_LOG_DEBUG,
		_("set/unset/has_key/get/map on int assoc"));
    assoc = lw6sys_assoc_new (NULL);
    lw6sys_assoc_set (&assoc, "a", (void *) &a);
    lw6sys_assoc_set (&assoc, "b", (void *) &b);
    lw6sys_assoc_set (&assoc, "c", (void *) &a);
    lw6sys_assoc_set (&assoc, "c", (void *) &c);
    lw6sys_assoc_unset (assoc, "b");
    lw6sys_assoc_map (assoc, &assoc_map_func, &ret);
    lw6sys_assoc_sort_and_map (assoc, &assoc_map_func, &ret);
    ret = ret && lw6sys_assoc_has_key (assoc, "a");
    ret = ret && !lw6sys_assoc_has_key (assoc, "b");
    ret = ret && !lw6sys_assoc_has_key (assoc, "this key does not exist");
    ret = ret && (*((int *) lw6sys_assoc_get (assoc, "a")) == 3);
    ret = ret && ((int *) lw6sys_assoc_get (assoc, "b") == NULL);
    ret = ret && (*((int *) lw6sys_assoc_get (assoc, "c")) == 7);
    lw6sys_assoc_free (assoc);
  }

  {
    char *str1, *str2, *str3;

    lw6sys_log (LW6SYS_LOG_DEBUG,
		_("testing free_func callback and dup on string assoc"));
    assoc = lw6sys_assoc_new (&lw6sys_free_callback);
    str1 = LW6SYS_MALLOC (5);
    strncpy (str1, "str1", 5);
    lw6sys_assoc_set (&assoc, "1", (void *) str1);
    str2 = LW6SYS_MALLOC (5);
    strncpy (str2, "str2", 5);
    lw6sys_assoc_set (&assoc, "2", (void *) str2);
    str3 = LW6SYS_MALLOC (5);
    strncpy (str3, "str3", 5);
    lw6sys_assoc_set (&assoc, "3", (void *) str3);
    lw6sys_assoc_unset (assoc, "1");
    lw6sys_assoc_unset (assoc, "3");
    assoc_copy = lw6sys_assoc_dup (assoc, (LW6SYS_DUP_FUNC) lw6sys_str_copy);
    ret = ret && !lw6sys_assoc_has_key (assoc_copy, "1");
    ret = ret && lw6sys_assoc_has_key (assoc_copy, "2");
    ret = ret && !lw6sys_assoc_has_key (assoc_copy, "3");
    ret = ret
      && strcmp ((char *) lw6sys_assoc_get (assoc_copy, "2"), "str2") == 0;
    lw6sys_assoc_free (assoc_copy);
    lw6sys_assoc_free (assoc);
  }

  {
    int a = 3, b = 5;
    lw6sys_list_t *keys;

    lw6sys_log (LW6SYS_LOG_DEBUG, _("testing keys on int assoc"));
    assoc = lw6sys_assoc_new (NULL);
    lw6sys_assoc_set (&assoc, "a", (void *) &a);
    lw6sys_assoc_set (&assoc, "b", (void *) &b);
    lw6sys_assoc_set (&assoc, "b", (void *) &b);
    keys = lw6sys_assoc_keys (assoc);
    lw6sys_assoc_free (assoc);
    ret = ret && lw6sys_list_length (keys) == 2;
    lw6sys_list_free (keys);
  }

  LW6SYS_TEST_FUNCTION_END;
  return ret;
}

/*
 * Test functions in build.c
 */
static int
test_build ()
{
  int ret = 1;
  LW6SYS_TEST_FUNCTION_BEGIN;

  {
    lw6sys_log (LW6SYS_LOG_DEBUG, _("build version is \"%s\""),
		lw6sys_build_get_version ());

    lw6sys_build_log_pedigree ();
  }

  LW6SYS_TEST_FUNCTION_END;
  return ret;
}

/*
 * Testing functions in checksum.c
 */
static int
test_checksum ()
{
  int ret = 1;
  LW6SYS_TEST_FUNCTION_BEGIN;

  {
    u_int32_t checksum = 0;

    lw6sys_checksum_update_int32 (&checksum, CHECKSUM_INT32_DATA);
    lw6sys_log (LW6SYS_LOG_DEBUG,
		_("checksum for %d is %08x (should be %08x)"),
		CHECKSUM_INT32_DATA, checksum, CHECKSUM_INT32_RESULT);
    if (checksum != CHECKSUM_INT32_RESULT)
      {
	ret = 0;
      }
  }

  LW6SYS_TEST_FUNCTION_END;
  return ret;
}

static void
rgb_to_hsv_to_rgb (char *label, lw6sys_color_8_t color_8)
{
  lw6sys_color_hsv_t color_hsv;
  lw6sys_color_8_t color_8_back;

  lw6sys_log (LW6SYS_LOG_DEBUG, _("%s init r=%d,g=%d,b=%d"), label, color_8.r,
	      color_8.g, color_8.b);
  lw6sys_color_rgb_to_hsv (&color_hsv, color_8);
  lw6sys_log (LW6SYS_LOG_DEBUG, _("%s HSV convert h=%.2f,s=%.2f,v=%.2f"),
	      label, color_hsv.h, color_hsv.s, color_hsv.v);
  color_8_back = lw6sys_color_hsv_to_rgb (&color_hsv);
  lw6sys_log (LW6SYS_LOG_DEBUG, _("%s RGB convert r=%d,g=%d,b=%d"), label,
	      color_8_back.r, color_8_back.g, color_8_back.b);
}

/*
 * Testing functions in color.c
 */
static int
test_color ()
{
  int ret = 1;
  LW6SYS_TEST_FUNCTION_BEGIN;

  {
    lw6sys_color_8_t color_8;
    lw6sys_color_f_t color_f;
    lw6sys_color_8_t color_8_average[3];
    float distance;

    lw6sys_log (LW6SYS_LOG_DEBUG,
		_("interpreting color \"%s\""), COLOR_ASCII);
    color_8 = lw6sys_color_a_to_8 (COLOR_ASCII);
    if (color_8.r != COLOR_8_R || color_8.g != COLOR_8_G
	|| color_8.b != COLOR_8_B || color_8.a != COLOR_8_A)
      {
	ret = 0;
      }

    lw6sys_color_8_to_f (&color_f, color_8);
    color_8 = lw6sys_color_f_to_8 (&color_f);

    rgb_to_hsv_to_rgb ("white", LW6SYS_COLOR_8_WHITE);
    rgb_to_hsv_to_rgb ("black", LW6SYS_COLOR_8_BLACK);
    rgb_to_hsv_to_rgb ("red", LW6SYS_COLOR_8_RED);
    rgb_to_hsv_to_rgb ("green", LW6SYS_COLOR_8_GREEN);
    rgb_to_hsv_to_rgb ("blue", LW6SYS_COLOR_8_BLUE);
    rgb_to_hsv_to_rgb ("test", color_8);

    color_8 =
      lw6sys_color_ponderate (LW6SYS_COLOR_8_RED, LW6SYS_COLOR_8_BLUE, 0.5f);
    lw6sys_log (LW6SYS_LOG_DEBUG,
		_("color between red & blue is r=%d,g=%d,b=%d"), color_8.r,
		color_8.g, color_8.b);

    color_8_average[0] = LW6SYS_COLOR_8_RED;
    color_8_average[1] = LW6SYS_COLOR_8_RED;
    color_8_average[2] = LW6SYS_COLOR_8_GREEN;
    color_8 = lw6sys_color_average (3, color_8_average);
    lw6sys_log (LW6SYS_LOG_DEBUG,
		_("color average of red+red+green is r=%d,g=%d,b=%d"),
		color_8.r, color_8.g, color_8.b);
    distance =
      lw6sys_color_distance (LW6SYS_COLOR_8_GREEN, LW6SYS_COLOR_8_BLUE);
    lw6sys_log (LW6SYS_LOG_DEBUG,
		_("distance between green and blue is %.2f"), distance);
  }

  LW6SYS_TEST_FUNCTION_END;
  return ret;
}

/*
 * Testing functions in convert.c
 */
static int
test_convert ()
{
  int ret = 1;
  LW6SYS_TEST_FUNCTION_BEGIN;

  {
    char *str;
    int i;
    float f;

    str = lw6sys_itoa (TEST_CONVERT_INT);
    if (str)
      {
	i = lw6sys_atoi (str);
	lw6sys_log (LW6SYS_LOG_DEBUG, _("int=%d -> str=\"%s\" -> int=%d"),
		    TEST_CONVERT_INT, str, i);
	if (i != TEST_CONVERT_INT)
	  {
	    ret = 0;
	  }
	LW6SYS_FREE (str);
      }
    else
      {
	ret = 0;
      }

    str = lw6sys_btoa (TEST_CONVERT_BOOL);
    if (str)
      {
	i = lw6sys_atob (str);
	lw6sys_log (LW6SYS_LOG_DEBUG, _("bool=%d -> str=\"%s\" -> bool=%d"),
		    TEST_CONVERT_BOOL, str, i);
	if (i != TEST_CONVERT_BOOL)
	  {
	    ret = 0;
	  }
	LW6SYS_FREE (str);
      }
    else
      {
	ret = 0;
      }

    str = lw6sys_ftoa (TEST_CONVERT_FLOAT);
    if (str)
      {
	f = lw6sys_atof (str);
	lw6sys_log (LW6SYS_LOG_DEBUG, _("float=%f -> str=\"%s\" -> float=%f"),
		    TEST_CONVERT_FLOAT, str, f);
	if (f != TEST_CONVERT_FLOAT)
	  {
	    lw6sys_log (LW6SYS_LOG_WARNING,
			_
			("%f!=%f: while this is not 100% blocking, floating point conversion on this computer is suspicious"),
			TEST_CONVERT_FLOAT, f);
	  }
	LW6SYS_FREE (str);
      }
    else
      {
	ret = 0;
      }
  }

  LW6SYS_TEST_FUNCTION_END;
  return ret;
}

/*
 * Testing functions in dump.c
 */
static int
test_dump ()
{
  int ret = 1;
  LW6SYS_TEST_FUNCTION_BEGIN;

  {
    char *user_dir = NULL;

    user_dir = lw6sys_get_default_user_dir ();
    if (user_dir)
      {
	lw6sys_create_dir (user_dir);
	lw6sys_dump_clear (user_dir);
	if (!lw6sys_dump (user_dir, TEST_CONTENT))
	  {
	    ret = 0;
	  }
	LW6SYS_FREE (user_dir);
      }
    else
      {
	ret = 0;
      }
  }

  LW6SYS_TEST_FUNCTION_END;
  return ret;
}

/*
 * Testing functions in env.c
 */
static int
test_env ()
{
  int ret = 1;
  LW6SYS_TEST_FUNCTION_BEGIN;

  {
    char *value;
    char *concat;

    concat = lw6sys_env_concat (TEST_ENV_CONCAT1, TEST_ENV_CONCAT2);
    if (concat)
      {
	lw6sys_log (LW6SYS_LOG_DEBUG, _("ENV concat is \"%s\""), concat);
	LW6SYS_FREE (concat);
      }
    else
      {
	ret = 0;
      }
    putenv (TEST_ENV_PUTENV);
    if (!lw6sys_env_exists (TEST_ENV_KEYWORD))
      {
	lw6sys_log (LW6SYS_LOG_WARNING, _("ENV \"%s\" doesn't exist"),
		    TEST_ENV_KEYWORD);
	ret = 0;
      }
    value = lw6sys_getenv (TEST_ENV_KEYWORD);
    if (value)
      {
	lw6sys_log (LW6SYS_LOG_DEBUG, _("ENV value for \"%s\" is \"%s\""),
		    TEST_ENV_KEYWORD, value);
	LW6SYS_FREE (value);
      }
    else
      {
	ret = 0;
      }
    value = lw6sys_get_username ();
    if (value)
      {
	lw6sys_log (LW6SYS_LOG_DEBUG, _("username=\"%s\""), value);
	LW6SYS_FREE (value);
      }
    else
      {
	ret = 0;
      }
    value = lw6sys_get_hostname ();
    if (value)
      {
	lw6sys_log (LW6SYS_LOG_DEBUG, _("hostname=\"%s\""), value);
	LW6SYS_FREE (value);
      }
    else
      {
	ret = 0;
      }
  }

  {
    lw6sys_list_t *splitted = NULL;
    char *data = NULL;

    splitted = lw6sys_env_split (TEST_ENV_SPLIT);
    if (splitted)
      {
	lw6sys_list_free (splitted);
      }
    else
      {
	ret = 0;
      }

    splitted = lw6sys_env_split (TEST_ENV_SPLIT);
    if (splitted)
      {
	while (splitted
	       && (data = (char *) lw6sys_lifo_pop (&splitted)) != NULL)
	  {
	    lw6sys_log (LW6SYS_LOG_DEBUG,
			_("splitted env \"%s\" contains \"%s\""),
			TEST_ENV_SPLIT, data);
	    LW6SYS_FREE (data);
	  }
	//lw6sys_list_free(splitted);
      }
    else
      {
	ret = 0;
      }
  }

  LW6SYS_TEST_FUNCTION_END;
  return ret;
}

/*
 * Testing functions in file.c
 */
static int
test_file ()
{
  int ret = 1;
  LW6SYS_TEST_FUNCTION_BEGIN;

  {
    char *content = NULL;

    if (lw6sys_write_file_content (TEST_FILENAME, TEST_CONTENT))
      {
	content = lw6sys_read_file_content (TEST_FILENAME);

	if (content && !strcmp (TEST_CONTENT, content))
	  {
	    lw6sys_log (LW6SYS_LOG_DEBUG, _("file \"%s\" contains \"%s\""),
			TEST_FILENAME, content);
	    LW6SYS_FREE (content);
	  }
	else
	  {
	    ret = 0;
	  }
      }
    else
      {
	ret = 0;
      }
  }

  LW6SYS_TEST_FUNCTION_END;
  return ret;
}

static int
test_hexa ()
{
  int ret = 1;
  LW6SYS_TEST_FUNCTION_BEGIN;

  {
    lw6sys_hexa_serializer_t *hexa_serializer;
    char *hexa_string = NULL;
    int64_t test_64;
    int32_t test_32;
    int16_t test_16;
    int8_t test_8;
    float test_float;
    char *test_str = NULL;
    lw6sys_wh_t test_wh = TEST_HEXA_WH;
    lw6sys_xy_t test_xy = TEST_HEXA_XY;
    lw6sys_color_8_t test_color = TEST_HEXA_COLOR;

    hexa_serializer = lw6sys_hexa_serializer_new ("");
    if (hexa_serializer)
      {
	lw6sys_hexa_serializer_rewind (hexa_serializer);
	ret = ret
	  && lw6sys_hexa_serializer_push_int64 (hexa_serializer,
						TEST_HEXA_INT64);
	ret = ret
	  && lw6sys_hexa_serializer_push_int32 (hexa_serializer,
						TEST_HEXA_INT32);
	ret = ret
	  && lw6sys_hexa_serializer_push_int16 (hexa_serializer,
						TEST_HEXA_INT16);
	ret = ret
	  && lw6sys_hexa_serializer_push_int8 (hexa_serializer,
					       TEST_HEXA_INT8);
	ret = ret
	  && lw6sys_hexa_serializer_push_float (hexa_serializer,
						TEST_HEXA_FLOAT);
	ret = ret
	  && lw6sys_hexa_serializer_push_str (hexa_serializer,
					      TEST_HEXA_STR1);
	ret = ret
	  && lw6sys_hexa_serializer_push_str (hexa_serializer,
					      TEST_HEXA_STR2);
	ret = ret
	  && lw6sys_hexa_serializer_push_wh (hexa_serializer, test_wh);
	ret = ret
	  && lw6sys_hexa_serializer_push_xy (hexa_serializer, test_xy);
	ret = ret
	  && lw6sys_hexa_serializer_push_color (hexa_serializer, test_color);
	if (ret)
	  {
	    hexa_string = lw6sys_hexa_serializer_as_string (hexa_serializer);
	    if (hexa_string)
	      {
		lw6sys_log (LW6SYS_LOG_DEBUG, _("hexa string is \"%s\""),
			    hexa_string);
		LW6SYS_FREE (hexa_string);
	      }

	    lw6sys_hexa_serializer_rewind (hexa_serializer);

	    if (lw6sys_hexa_serializer_eof (hexa_serializer))
	      {
		lw6sys_log (LW6SYS_LOG_WARNING,
			    _("unexpected serializer EOF"));
		ret = 0;
	      }

	    if (lw6sys_hexa_serializer_pop_int64 (hexa_serializer, &test_64))
	      {
		lw6sys_log (LW6SYS_LOG_DEBUG, _("int64 loaded, value=%d"),
			    (int) test_64);
	      }
	    else
	      {
		ret = 0;
	      }
	    if (lw6sys_hexa_serializer_pop_int32 (hexa_serializer, &test_32))
	      {
		lw6sys_log (LW6SYS_LOG_DEBUG, _("int32 loaded, value=%d"),
			    (int) test_32);
	      }
	    else
	      {
		ret = 0;
	      }
	    if (lw6sys_hexa_serializer_pop_int16 (hexa_serializer, &test_16))
	      {
		lw6sys_log (LW6SYS_LOG_DEBUG, _("int16 loaded, value=%d"),
			    (int) test_16);
	      }
	    else
	      {
		ret = 0;
	      }
	    if (lw6sys_hexa_serializer_pop_int8 (hexa_serializer, &test_8))
	      {
		lw6sys_log (LW6SYS_LOG_DEBUG, _("int8 loaded, value=%d"),
			    (int) test_8);
	      }
	    else
	      {
		ret = 0;
	      }
	    if (lw6sys_hexa_serializer_pop_float
		(hexa_serializer, &test_float))
	      {
		lw6sys_log (LW6SYS_LOG_DEBUG, _("float loaded, value=%f"),
			    test_float);
	      }
	    else
	      {
		ret = 0;
	      }
	    if (lw6sys_hexa_serializer_pop_str (hexa_serializer, &test_str))
	      {
		lw6sys_log (LW6SYS_LOG_DEBUG, _("str loaded, value=\"%s\""),
			    test_str);
		LW6SYS_FREE (test_str);
	      }
	    else
	      {
		ret = 0;
	      }
	    if (lw6sys_hexa_serializer_pop_str (hexa_serializer, &test_str))
	      {
		lw6sys_log (LW6SYS_LOG_DEBUG, _("str loaded, value=\"%s\""),
			    test_str);
		LW6SYS_FREE (test_str);
	      }
	    else
	      {
		ret = 0;
	      }
	    if (lw6sys_hexa_serializer_pop_wh (hexa_serializer, &test_wh))
	      {
		lw6sys_log (LW6SYS_LOG_DEBUG, _("wh loaded, value=%dx%d"),
			    (int) test_wh.w, (int) test_wh.h);
	      }
	    else
	      {
		ret = 0;
	      }
	    if (lw6sys_hexa_serializer_pop_xy (hexa_serializer, &test_xy))
	      {
		lw6sys_log (LW6SYS_LOG_DEBUG, _("xy loaded, value=%d,%d"),
			    (int) test_xy.x, (int) test_xy.y);
	      }
	    else
	      {
		ret = 0;
	      }
	    if (lw6sys_hexa_serializer_pop_color
		(hexa_serializer, &test_color))
	      {
		lw6sys_log (LW6SYS_LOG_DEBUG,
			    _("color loaded, value=#%02x%02x%02x%02x"),
			    (int) test_color.r, (int) test_color.g,
			    (int) test_color.b, (int) test_color.a);
	      }
	    else
	      {
		ret = 0;
	      }
	    if (lw6sys_hexa_serializer_eof (hexa_serializer))
	      {
		lw6sys_log (LW6SYS_LOG_DEBUG, _("serializer EOF"));
	      }
	    else
	      {
		ret = 0;
	      }
	  }

	lw6sys_hexa_serializer_free (hexa_serializer);
      }
    else
      {
	ret = 0;
      }
  }

  LW6SYS_TEST_FUNCTION_END;
  return ret;
}

/*
 * Testing functions in i18n.c
 */
static int
test_i18n ()
{
  int ret = 1;
  LW6SYS_TEST_FUNCTION_BEGIN;

  {
    char *utf8_str;
    int local_ret = 0;

    utf8_str = lw6sys_locale_to_utf8 (ASCII_STR);
    if (utf8_str)
      {
	if (strcmp (utf8_str, ASCII_STR) == 0)
	  {
	    local_ret = 1;
	  }

	LW6SYS_FREE (utf8_str);
      }

    ret = ret && local_ret;
  }

  LW6SYS_TEST_FUNCTION_END;
  return ret;
}

/*
 * Testing functions in keyword.c
 */
static int
test_keyword ()
{
  int ret = 1;
  LW6SYS_TEST_FUNCTION_BEGIN;

  {
    char *str;

    str = lw6sys_keyword_as_key (TEST_KEYWORD_AS);
    if (str)
      {
	lw6sys_log (LW6SYS_LOG_DEBUG, _("keyword as key is \"%s\""), str);
	LW6SYS_FREE (str);
      }
    str = lw6sys_keyword_as_arg (TEST_KEYWORD_AS);
    if (str)
      {
	lw6sys_log (LW6SYS_LOG_DEBUG, _("keyword as arg is \"%s\""), str);
	LW6SYS_FREE (str);
      }
    str = lw6sys_keyword_as_env (TEST_KEYWORD_AS);
    if (str)
      {
	lw6sys_log (LW6SYS_LOG_DEBUG, _("keyword as env is \"%s\""), str);
	LW6SYS_FREE (str);
      }
    str = lw6sys_keyword_as_xml (TEST_KEYWORD_AS);
    if (str)
      {
	lw6sys_log (LW6SYS_LOG_DEBUG, _("keyword as xml is \"%s\""), str);
	LW6SYS_FREE (str);
      }
  }

  LW6SYS_TEST_FUNCTION_END;
  return ret;
}

/*
 * Used to test lw6sys_list_map.
 */
static void
list_map_func (void *func_data, void *data)
{
  int *ret = (int *) func_data;
  int *i = (int *) data;

  lw6sys_log (LW6SYS_LOG_DEBUG, _("list item ret=%d, data=%d"), *ret, *i);
}

/*
 * Testing functions in list.c
 */
static int
test_list ()
{
  int ret = 1;
  lw6sys_list_t *list;
  lw6sys_list_t *list_copy;
  LW6SYS_TEST_FUNCTION_BEGIN;

  {
    lw6sys_log (LW6SYS_LOG_DEBUG, _("new/delete/is_empty on list"));
    list = lw6sys_list_new (NULL);
    ret = ret && lw6sys_list_is_empty (list);
    lw6sys_list_free (list);
  }

  {
    int a = 3, b = 5, c = 7;

    lw6sys_log (LW6SYS_LOG_DEBUG, _("lifo push/pop on int list"));
    list = lw6sys_list_new (NULL);
    lw6sys_lifo_push (&list, (void *) &a);
    lw6sys_lifo_push (&list, (void *) &b);
    lw6sys_lifo_push (&list, (void *) &c);
    lw6sys_list_map (list, &list_map_func, &ret);
    ret = ret && lw6sys_list_length (list) == 3;
    ret = ret && (*((int *) lw6sys_lifo_pop (&list)) == c);
    ret = ret && (*((int *) lw6sys_lifo_pop (&list)) == b);
    ret = ret && (*((int *) lw6sys_lifo_pop (&list)) == a);
    lw6sys_list_free (list);
  }

  {
    int a = 2, b = 4, c = 6, d = 8;

    lw6sys_log (LW6SYS_LOG_DEBUG, _("fifo push/pop on int list"));
    list = lw6sys_list_new (NULL);
    lw6sys_fifo_push (&list, (void *) &a);
    lw6sys_fifo_push (&list, (void *) &b);
    lw6sys_fifo_push (&list, (void *) &c);
    lw6sys_fifo_push (&list, (void *) &d);
    lw6sys_list_map (list, &list_map_func, &ret);
    ret = ret && lw6sys_list_length (list) == 4;
    ret = ret && (*((int *) lw6sys_fifo_pop (&list)) == a);
    ret = ret && (*((int *) lw6sys_fifo_pop (&list)) == b);
    ret = ret && (*((int *) lw6sys_fifo_pop (&list)) == c);
    ret = ret && (*((int *) lw6sys_fifo_pop (&list)) == d);
    lw6sys_list_free (list);
  }

  {
    int a = 32, b = 64, c = 128, d = 256, e = 512;

    lw6sys_log (LW6SYS_LOG_DEBUG, _("front/back push/pop on int list"));
    list = lw6sys_list_new (NULL);
    lw6sys_list_push_front (&list, (void *) &a);
    lw6sys_list_push_front (&list, (void *) &b);
    lw6sys_list_push_back (&list, (void *) &c);
    lw6sys_list_push_back (&list, (void *) &d);
    lw6sys_list_push_back (&list, (void *) &e);
    lw6sys_list_map (list, &list_map_func, &ret);
    ret = ret && lw6sys_list_length (list) == 5;
    ret = ret && (*((int *) lw6sys_list_pop_back (&list)) == e);
    ret = ret && (*((int *) lw6sys_list_pop_back (&list)) == d);
    ret = ret && (*((int *) lw6sys_list_pop_front (&list)) == b);
    ret = ret && (*((int *) lw6sys_list_pop_front (&list)) == a);
    ret = ret && (*((int *) lw6sys_list_pop_front (&list)) == c);
    lw6sys_list_free (list);
  }

  {
    char *str1, *str2, *str3, *str4;

    lw6sys_log (LW6SYS_LOG_DEBUG,
		_("testing free_func callback and dup on string list"));
    list = lw6sys_list_new (&lw6sys_free_callback);
    str1 = lw6sys_str_copy ("str1");
    lw6sys_lifo_push (&list, (void *) str1);
    str2 = lw6sys_str_copy ("str2");
    lw6sys_fifo_push (&list, (void *) str2);
    str3 = lw6sys_str_copy ("str3");
    lw6sys_lifo_push (&list, (void *) str3);
    str4 = lw6sys_str_copy ("str4");
    lw6sys_fifo_push (&list, (void *) str4);
    list_copy = lw6sys_list_dup (list, (LW6SYS_DUP_FUNC) lw6sys_str_copy);
    ret = ret && lw6sys_list_length (list_copy) == 4;
    lw6sys_list_free (list_copy);
    lw6sys_list_free (list);
  }

  {
    lw6sys_log (LW6SYS_LOG_DEBUG, _("testing final pop behavior"));

    list = lw6sys_list_new (NULL);
    if (list)
      {
	lw6sys_list_pop_front (&list);
	if (list)
	  {
	    lw6sys_log (LW6SYS_LOG_WARNING,
			_("final pop front didn't free list"));
	    lw6sys_list_free (list);
	    ret = 0;
	  }
      }

    list = lw6sys_list_new (NULL);
    if (list)
      {
	lw6sys_list_pop_back (&list);
	if (list)
	  {
	    lw6sys_log (LW6SYS_LOG_WARNING,
			_("final pop back didn't free list"));
	    lw6sys_list_free (list);
	    ret = 0;
	  }
      }
  }

  LW6SYS_TEST_FUNCTION_END;
  return ret;
}

/*
 * Testing functions in log.c
 */
static int
test_log ()
{
  int ret = 1;
  LW6SYS_TEST_FUNCTION_BEGIN;

  lw6sys_log (LW6SYS_LOG_DEBUG, _("testing log_debug %s=%d"), "toto", 0);
  lw6sys_log (LW6SYS_LOG_INFO, _("testing log_info %s=%d"), "toto", 0);
  /*
   * We do not test WARNING & ERROR to avoid confusion...
   */

  LW6SYS_TEST_FUNCTION_END;
  return ret;
}

/*
 * Testing functions in mem.c
 */
static int
test_mem ()
{
  int ret = 1;
  char *ptr = NULL;
  LW6SYS_TEST_FUNCTION_BEGIN;

  lw6sys_log (LW6SYS_LOG_DEBUG,
	      _("testing MALLOC/FREE (%d bytes)"), MALLOC_SIZE);
  ptr = (char *) LW6SYS_MALLOC (MALLOC_SIZE * sizeof (char));
  ret = ret && (ptr != NULL);
  LW6SYS_FREE (ptr);
  lw6sys_log (LW6SYS_LOG_DEBUG, _("%d megs"), lw6sys_megabytes_available ());
  if (lw6sys_is_big_endian ())
    {
      lw6sys_log (LW6SYS_LOG_DEBUG, _("big endian"));
    }
  if (lw6sys_is_little_endian ())
    {
      lw6sys_log (LW6SYS_LOG_DEBUG, _("little endian"));
    }

  LW6SYS_TEST_FUNCTION_END;
  return ret;
}

/*
 * Testing functions in mutex.c
 */
static int
test_mutex ()
{
  int ret = 0;
  void *mutex = NULL;
  LW6SYS_TEST_FUNCTION_BEGIN;

  {
    lw6sys_log (LW6SYS_LOG_DEBUG, _("testing mutex functions"));
    mutex = lw6sys_mutex_create ();
    if (mutex)
      {
	if (lw6sys_mutex_lock (mutex))
	  {
	    if (lw6sys_mutex_trylock (mutex))
	      {
		lw6sys_log (LW6SYS_LOG_WARNING,
			    _
			    ("strange, trylock on locked mutex should return 0"));
	      }
	    else
	      {
		if (lw6sys_mutex_unlock (mutex))
		  {
		    ret = 1;
		  }
	      }
	  }
	lw6sys_mutex_destroy (mutex);
      }
  }

  LW6SYS_TEST_FUNCTION_END;
  return ret;
}

/*
 * Testing options functions
 */
static int
test_options ()
{
  int ret = 1;
  LW6SYS_TEST_FUNCTION_BEGIN;

  {
    char *path;

    path = lw6sys_get_default_user_dir ();
    if (path)
      {
	lw6sys_log (LW6SYS_LOG_DEBUG, _("default user dir is \"%s\""), path);
	LW6SYS_FREE (path);
      }
    else
      {
	ret = 0;
      }

    path = lw6sys_get_default_prefix ();
    if (path)
      {
	lw6sys_log (LW6SYS_LOG_DEBUG, _("default prefix is \"%s\""), path);
	LW6SYS_FREE (path);
      }
    else
      {
	ret = 0;
      }

    lw6sys_options_log_defaults ();
  }

  {
    char *path;
    int argc = TEST_ARGC;
    char *argv[TEST_ARGC] =
      { TEST_ARGV0, TEST_ARGV1, TEST_ARGV2, TEST_ARGV3 };

    path = lw6sys_get_user_dir (argc, argv);
    if (path)
      {
	lw6sys_log (LW6SYS_LOG_DEBUG, _("user dir is \"%s\""), path);
	LW6SYS_FREE (path);
      }
    else
      {
	ret = 0;
      }

    path = lw6sys_get_prefix (argc, argv);
    if (path)
      {
	lw6sys_log (LW6SYS_LOG_DEBUG, _("prefix is \"%s\""), path);
	LW6SYS_FREE (path);
      }
    else
      {
	ret = 0;
      }

    lw6sys_options_log (argc, argv);
  }

  LW6SYS_TEST_FUNCTION_END;
  return ret;
}

/*
 * Testing functions in nop.c
 */
static int
test_nop ()
{
  int ret = 0;
  LW6SYS_TEST_FUNCTION_BEGIN;

  {
    ret = lw6sys_true () && !lw6sys_false ();
  }

  LW6SYS_TEST_FUNCTION_END return ret;
}

/*
 * Testing functions in path.c
 */
static int
test_path ()
{
  int ret = 1;
  LW6SYS_TEST_FUNCTION_BEGIN;

  {
    char *path;
    int relative;

    lw6sys_log (LW6SYS_LOG_DEBUG, _("add/strip slash"));
    path = lw6sys_path_add_slash (PATH_WITHOUT_SLASH);
    if (path)
      {
	ret = ret && (strcmp (path, PATH_WITH_SLASH) == 0);
	LW6SYS_FREE (path);
      }
    path = lw6sys_path_strip_slash (PATH_WITH_SLASH);
    if (path)
      {
	ret = ret && (strcmp (path, PATH_WITHOUT_SLASH) == 0);
	LW6SYS_FREE (path);
      }
    lw6sys_log (LW6SYS_LOG_DEBUG, _("concat path"));
    path = lw6sys_path_concat (PATH_WITH_SLASH, PATH_WITH_SLASH);
    if (path)
      {
	lw6sys_log (LW6SYS_LOG_DEBUG, _("concatenated path = \"%s\""), path);
	ret = ret && (strcmp (path, PATH_CONCAT) == 0);
	LW6SYS_FREE (path);
      }
    relative = lw6sys_path_is_relative (PATH_RELATIVE);
    if (!relative)
      {
	lw6sys_log (LW6SYS_LOG_WARNING,
		    _("path \"%s\" not considered relative"), PATH_RELATIVE);
	ret = 0;
      }
    path = lw6sys_path_parent (PATH_CONCAT);
    if (path)
      {
	lw6sys_log (LW6SYS_LOG_DEBUG, _("parent path = \"%s\""), path);
	ret = ret && (strcmp (path, PATH_CONCAT_PARENT) == 0);
	LW6SYS_FREE (path);
      }
    path = lw6sys_path_unparent (PATH_RELATIVE);
    if (path)
      {
	lw6sys_log (LW6SYS_LOG_DEBUG, _("unparent path = \"%s\""), path);
	ret = ret && (strcmp (path, PATH_RELATIVE_UNPARENT) == 0);
	LW6SYS_FREE (path);
      }
    path = lw6sys_path_unparent_no_malloc (PATH_RELATIVE);
    lw6sys_log (LW6SYS_LOG_DEBUG, _("unparent path = \"%s\""), path);
    ret = ret && (strcmp (path, PATH_RELATIVE_UNPARENT) == 0);
  }

  LW6SYS_TEST_FUNCTION_END;
  return ret;
}

/*
 * Testing functions in sort.c
 */
static int
test_sort ()
{
  int ret = 1;
  LW6SYS_TEST_FUNCTION_BEGIN;

  {
    lw6sys_list_t *list = NULL;
    int array[TEST_SORT_LENGTH] = TEST_SORT_INT;
    int i;

    list = lw6sys_list_new (NULL);
    if (list)
      {
	for (i = 0; i < TEST_SORT_LENGTH; ++i)
	  {
	    lw6sys_lifo_push (&list, (void *) (array + i));
	  }
	lw6sys_sort (&list, lw6sys_sort_int_callback);
	if (list)
	  {
	    if ((*((int *) (list->data))) == TEST_SORT_INT_MIN)
	      {
		lw6sys_log (LW6SYS_LOG_DEBUG, _("min sorted value is %d"),
			    TEST_SORT_INT_MIN);
	      }
	    else
	      {
		lw6sys_log (LW6SYS_LOG_WARNING,
			    _("min sorted value is %d and should be %d"),
			    *((int *) (list->data)), TEST_SORT_INT_MIN);
		ret = 0;
	      }
	  }
	lw6sys_sort (&list, lw6sys_sort_int_desc_callback);
	if (list)
	  {
	    if ((*((int *) (list->data))) == TEST_SORT_INT_MAX)
	      {
		lw6sys_log (LW6SYS_LOG_DEBUG, _("max sorted value is %d"),
			    TEST_SORT_INT_MAX);
	      }
	    else
	      {
		lw6sys_log (LW6SYS_LOG_WARNING,
			    _("max sorted value is %d and should be %d"),
			    *((int *) (list->data)), TEST_SORT_INT_MAX);
		ret = 0;
	      }
	  }
	if (list)
	  {
	    lw6sys_list_free (list);
	  }
	else
	  {
	    ret = 0;
	  }
      }
    else
      {
	ret = 0;
      }
  }

  {
    lw6sys_list_t *list = NULL;
    float array[TEST_SORT_LENGTH] = TEST_SORT_FLOAT;
    int i;

    list = lw6sys_list_new (NULL);
    if (list)
      {
	for (i = 0; i < TEST_SORT_LENGTH; ++i)
	  {
	    lw6sys_lifo_push (&list, (void *) (array + i));
	  }
	lw6sys_sort (&list, lw6sys_sort_float_callback);
	if (list)
	  {
	    if ((*((float *) (list->data))) == TEST_SORT_FLOAT_MIN)
	      {
		lw6sys_log (LW6SYS_LOG_DEBUG, _("min sorted value is %f"),
			    TEST_SORT_FLOAT_MIN);
	      }
	    else
	      {
		lw6sys_log (LW6SYS_LOG_WARNING,
			    _("min sorted value is %f and should be %f"),
			    *((float *) (list->data)), TEST_SORT_FLOAT_MIN);
		ret = 0;
	      }
	  }
	lw6sys_sort (&list, lw6sys_sort_float_desc_callback);
	if (list)
	  {
	    if ((*((float *) (list->data))) == TEST_SORT_FLOAT_MAX)
	      {
		lw6sys_log (LW6SYS_LOG_DEBUG, _("max sorted value is %f"),
			    TEST_SORT_FLOAT_MAX);
	      }
	    else
	      {
		lw6sys_log (LW6SYS_LOG_WARNING,
			    _("max sorted value is %f and should be %f"),
			    *((float *) (list->data)), TEST_SORT_FLOAT_MAX);
		ret = 0;
	      }
	  }
	if (list)
	  {
	    lw6sys_list_free (list);
	  }
	else
	  {
	    ret = 0;
	  }
      }
    else
      {
	ret = 0;
      }
  }

  {
    lw6sys_list_t *list = NULL;
    char *array[TEST_SORT_LENGTH] = TEST_SORT_STR;
    int i;

    list = lw6sys_list_new (NULL);
    if (list)
      {
	for (i = 0; i < TEST_SORT_LENGTH; ++i)
	  {
	    lw6sys_lifo_push (&list, (void *) array[i]);
	  }
	lw6sys_sort (&list, lw6sys_sort_str_callback);
	if (list)
	  {
	    if (!strcmp ((char *) (list->data), TEST_SORT_STR_MIN))
	      {
		lw6sys_log (LW6SYS_LOG_DEBUG, _("min sorted value is \"%s\""),
			    TEST_SORT_STR_MIN);
	      }
	    else
	      {
		lw6sys_log (LW6SYS_LOG_WARNING,
			    _
			    ("min sorted value is \"%s\" and should be \"%s\""),
			    (char *) (list->data), TEST_SORT_STR_MIN);
		ret = 0;
	      }
	  }
	lw6sys_sort (&list, lw6sys_sort_str_desc_callback);
	if (list)
	  {
	    if (!strcmp ((char *) (list->data), TEST_SORT_STR_MAX))
	      {
		lw6sys_log (LW6SYS_LOG_DEBUG, _("max sorted value is \"%s\""),
			    TEST_SORT_STR_MAX);
	      }
	    else
	      {
		lw6sys_log (LW6SYS_LOG_WARNING,
			    _
			    ("max sorted value is \"%s\" and should be \"%s\""),
			    (char *) (list->data), TEST_SORT_STR_MAX);
		ret = 0;
	      }
	  }
	if (list)
	  {
	    lw6sys_list_free (list);
	  }
	else
	  {
	    ret = 0;
	  }
      }
    else
      {
	ret = 0;
      }
  }

  LW6SYS_TEST_FUNCTION_END;
  return ret;
}

/*
 * Testing functions in str.c
 */
static int
test_str ()
{
  int ret = 1;
  LW6SYS_TEST_FUNCTION_BEGIN;

  {
    char *str;

    lw6sys_log (LW6SYS_LOG_DEBUG, _("testing is_blank on \"%s\""), BLANK_STR);
    ret = ret && lw6sys_str_is_blank (BLANK_STR);
    str =
      lw6sys_str_reformat (TEST_REFORMAT_STR1, TEST_REFORMAT_PREFIX,
			   TEST_REFORMAT_COLUMNS);
    if (str)
      {
	lw6sys_log (LW6SYS_LOG_DEBUG, _("reformatted string is \"%s\""), str);
	LW6SYS_FREE (str);
      }
    else
      {
	ret = 0;
      }

    str =
      lw6sys_str_reformat (TEST_REFORMAT_STR2, TEST_REFORMAT_PREFIX,
			   TEST_REFORMAT_COLUMNS);
    if (str)
      {
	lw6sys_log (LW6SYS_LOG_DEBUG, _("reformatted string is \"%s\""), str);
	LW6SYS_FREE (str);
      }
    else
      {
	ret = 0;
      }
  }

  LW6SYS_TEST_FUNCTION_END;
  return ret;
}

/*
 * Testing functions in sdl.c
 */
static int
test_sdl ()
{
  int ret = 1;
  LW6SYS_TEST_FUNCTION_BEGIN;

  {
    lw6sys_log (LW6SYS_LOG_DEBUG, _("register/unregister SDL"));
    ret = ret && lw6sys_sdl_register () && lw6sys_sdl_unregister ();
  }

  LW6SYS_TEST_FUNCTION_END;
  return ret;
}

/*
 * Testing functions in serial.c
 */
static int
test_serial ()
{
  int ret = 1;
  LW6SYS_TEST_FUNCTION_BEGIN;

  {
    int64_t test64 = 0;
    int32_t test32 = 0;
    int16_t test16 = 0;
    unsigned char buffer[LW6SYS_SIZEOF_INT64];

    lw6sys_serialize_int64 (buffer, TEST_SERIALIZE);
    test64 = lw6sys_unserialize_int64 (buffer);
    if (test64 != TEST_SERIALIZE)
      {
	lw6sys_log (LW6SYS_LOG_WARNING,
		    _("serialize failed on 64-bit integer (%d/%d)"),
		    (int) test64, TEST_SERIALIZE);
	ret = 0;
      }

    lw6sys_serialize_int32 (buffer, TEST_SERIALIZE);
    test32 = lw6sys_unserialize_int32 (buffer);
    if (test32 != TEST_SERIALIZE)
      {
	lw6sys_log (LW6SYS_LOG_WARNING,
		    _("serialize failed on 32-bit integer (%d/%d)"),
		    (int) test32, TEST_SERIALIZE);
	ret = 0;
      }

    lw6sys_serialize_int16 (buffer, TEST_SERIALIZE);
    test16 = lw6sys_unserialize_int16 (buffer);
    if (test16 != TEST_SERIALIZE)
      {
	lw6sys_log (LW6SYS_LOG_WARNING,
		    _("serialize failed on 16-bit integer (%d/%d)"),
		    (int) test16, TEST_SERIALIZE);
	ret = 0;
      }
  }

  LW6SYS_TEST_FUNCTION_END;
  return ret;
}

static void
thread_func (void *callback_data)
{
  int i = 0;
  char *text = NULL;

  text = (char *) callback_data;
  lw6sys_log (LW6SYS_LOG_DEBUG, _("thread_callback text=\"%s\""), text);
  for (i = 0; i < THREAD_N; ++i)
    {
      lw6sys_log (LW6SYS_LOG_DEBUG, _("thread_callback step %d"), i + 1);
      lw6sys_sleep (THREAD_SLEEP_CALLBACK);
    }
}

static void
thread_join (void *callback_data)
{
  /*
   * A more complex example would process more complex
   * freeing of resources here.
   */
  LW6SYS_FREE (callback_data);
}

/*
 * Testing functions in thread.c
 */
static int
test_thread ()
{
  int ret = 1;
  LW6SYS_TEST_FUNCTION_BEGIN;

  {
    void *thread_handler = NULL;
    char *text = lw6sys_str_copy (THREAD_TEXT);
    int i = 0;

    ret = 0;
    if (text)
      {
	thread_handler =
	  lw6sys_thread_create (&thread_func, &thread_join, (void *) text, 0);
	if (thread_handler)
	  {
	    for (i = 0; i < THREAD_N; ++i)
	      {
		lw6sys_log (LW6SYS_LOG_DEBUG,
			    _("thread_main step %d, callback_done=%d"), i + 1,
			    lw6sys_thread_is_callback_done (thread_handler));
		lw6sys_sleep (THREAD_SLEEP_MAIN);
	      }

	    lw6sys_thread_join (thread_handler);
	    ret = 1;
	  }
      }
  }

  LW6SYS_TEST_FUNCTION_END;
  return ret;
}

/*
 * Testing functions in time.c
 */
static int
test_time ()
{
  int ret = 1;
  LW6SYS_TEST_FUNCTION_BEGIN;

  lw6sys_log (LW6SYS_LOG_DEBUG, _("timestamp %lld"), lw6sys_timestamp ());
  lw6sys_log (LW6SYS_LOG_DEBUG, _("sleep %d seconds"), SLEEP_TIME);
  lw6sys_sleep (SLEEP_TIME);

  LW6SYS_TEST_FUNCTION_END;
  return ret;
}

/*
 * Launch tests
 */
int
lw6sys_test ()
{
  int ret = 0;

  ret = test_arg () && test_assoc () && test_build () && test_checksum ()
    && test_color () && test_convert () && test_dump () && test_env ()
    && test_file () && test_hexa () && test_i18n () && test_keyword ()
    && test_list () && test_log () && test_mem () && test_mutex ()
    && test_options () && test_nop () && test_path () && test_sdl ()
    && test_serial () && test_sort () && test_str () && test_thread ()
    && test_time ();

  return ret;
}
